<?php
namespace Elementor;
/**
 * @package     WordPress
 * @subpackage  Recent Posts Elementor Addon
 * @author      support@themegum.com
 * @since       1.0.0
*/
defined('GUM_RCPOST_ELEMENTOR_DIR') or die();

use Elementor\Widget_Base;
use Elementor\Controls_Manager;
use Elementor\Group_Control_Border;
use Elementor\Group_Control_Typography;
use Elementor\Icons_Manager;
use WP_Query;

class Gum_Elementor_Widget_Recent_posts extends Widget_Base {

  public function __construct( $data = [], $args = null ) {
    parent::__construct( $data, $args );

    $is_type_instance = $this->is_type_instance();

    if ( ! $is_type_instance && null === $args ) {
      throw new \Exception( '`$args` argument is required when initializing a full widget instance.' );
    }

    add_action( 'elementor/element/before_section_start', [ $this, 'enqueue_script' ] );

    if ( $is_type_instance ) {
      $this->_register_skins();

      $widget_name = $this->get_name();

      /**
       * Widget skin init.
       *
       * Fires when Elementor widget is being initialized.
       *
       * The dynamic portion of the hook name, `$widget_name`, refers to the widget name.
       *
       * @since 1.0.0
       *
       * @param Widget_Base $this The current widget.
       */
      do_action( "elementor/widget/{$widget_name}/skins_init", $this );
    }
  }


  /**
   * Get widget name.
   *
   *
   * @since 1.0.0
   * @access public
   *
   * @return string Widget name.
   */
  public function get_name() {
    return 'gum_posts_list_pro';
  }

  /**
   * Get widget title.
   *
   *
   * @since 1.0.0
   * @access public
   *
   * @return string Widget title.
   */
  public function get_title() {

    return esc_html__( 'Recent Posts Pro', 'recent-posts-elementor-addon' );
  }

  /**
   * Get widget icon.
   *
   *
   * @since 1.0.0
   * @access public
   *
   * @return string Widget icon.
   */
  public function get_icon() {
    return 'eicon-post-list gum-kit';
  }

  /**
   * Get widget categories.
   *
   *
   * @since 1.0.0
   * @access public
   *
   * @return array Widget categories.
   */
  public function get_categories() {
    return [ 'temegum' ];
  }

  public function get_keywords() {
    return [ 'wordpress', 'widget', 'post','recent','blog' ];
  }

  protected function _register_controls() {




  $this->start_controls_section(
    'section_title',
    [
      'label' => esc_html__( 'Layout', 'recent-posts-elementor-addon' ),
    ]
  );


  $this->add_control(
    'show_image',
    [
      'label' => esc_html__( 'Show Image', 'recent-posts-elementor-addon' ),
      'type' => Controls_Manager::SWITCHER,
      'label_off' => esc_html__( 'No', 'recent-posts-elementor-addon' ),
      'label_on' => esc_html__( 'Yes', 'recent-posts-elementor-addon' ),
      'default' => '',
    ]
  );


  $this->add_control(
    'image_position',
    [
      'label' => esc_html__( 'Image Position', 'recent-posts-elementor-addon' ),
      'type' => Controls_Manager::CHOOSE,
      'options' => [
        'left' => [
          'title' => esc_html__( 'Left', 'recent-posts-elementor-addon' ),
          'icon' => 'eicon-h-align-left',
        ],
        'top' => [
          'title' => esc_html__( 'Top', 'recent-posts-elementor-addon' ),
          'icon' => 'eicon-v-align-top',
        ],
        'right' => [
          'title' => esc_html__( 'Right', 'recent-posts-elementor-addon' ),
          'icon' => 'eicon-h-align-right',
        ],
      ],
      'default' => 'top',
      'condition' => [
        'show_image!' => '',
      ],
    ]
  );

  $this->add_control(
    'image_totop',
    [
      'label' => esc_html__( 'Ignore on Mobile', 'recent-posts-elementor-addon' ),
      'type' => Controls_Manager::SWITCHER,
      'label_off' => esc_html__( 'No', 'recent-posts-elementor-addon' ),
      'label_on' => esc_html__( 'Yes', 'recent-posts-elementor-addon' ),
      'default' => 'yes',
      'description' => esc_html__( 'Image changed to top on mobile', 'recent-posts-elementor-addon' ),
      'condition' => [
        'show_image!' => '',
        'image_position!' => 'top',
      ],
    ]
  );

  $this->add_group_control(
    Group_Control_Image_Size::get_type(),
    [
      'name' => 'thumbnail', 
      'default' => 'medium',
      'condition' => [
        'show_image!' => '',
      ],
    ]
  );



  $this->add_control(
    'tag',
    [
      'label' => esc_html__( 'Title HTML Tag', 'recent-posts-elementor-addon' ),
      'type' => Controls_Manager::SELECT,
      'options' => [
        'h2' => 'H2',
        'h3' => 'H3',
        'h4' => 'H4',
        'h5' => 'H5',
        'h6' => 'H6',
        'div' => 'div',
      ],
      'default' => 'h4',
    ]
  );

  $this->add_control(
    'show_excerpt',
    [
      'label' => esc_html__( 'Show Post Excerpt', 'recent-posts-elementor-addon' ),
      'type' => Controls_Manager::SWITCHER,
      'label_off' => esc_html__( 'No', 'recent-posts-elementor-addon' ),
      'label_on' => esc_html__( 'Yes', 'recent-posts-elementor-addon' ),
      'default' => 'yes',
    ]
  );

   $this->add_control(
      'post_content_word',
      [
        'label'     => esc_html__( 'Word Count', 'recent-posts-elementor-addon' ),
        'type'      => Controls_Manager::NUMBER,
        'default'   => '',
        'condition' => [
          'show_excerpt' => 'yes',
        ],
      ]
    );

   $this->add_control(
      'post_content_sufix',
      [
        'label'     => esc_html__( 'Suffix', 'recent-posts-elementor-addon' ),
        'type'      => Controls_Manager::TEXT,
        'default'   => '',
        'condition' => [
          'show_excerpt' => 'yes',
          'post_content_word!' => ''
        ],
      ]
    );   

    $this->add_control(
      'show_readmore',
      [
        'label' => esc_html__( 'Show Readmore', 'recent-posts-elementor-addon' ),
        'type' => Controls_Manager::SWITCHER,
        'label_off' => esc_html__( 'No', 'recent-posts-elementor-addon' ),
        'label_on' => esc_html__( 'Yes', 'recent-posts-elementor-addon' ),
        'default' => '',
      ]
    );

    $this->add_control(
      'readmore_label',
      [
        'label' => esc_html__( 'Label', 'recent-posts-elementor-addon' ),
        'type' => Controls_Manager::TEXT,
        'default' => esc_html__( 'Readmore', 'recent-posts-elementor-addon' ),
        'label_block' => true,
        'condition' => [
          'show_readmore' => 'yes',
        ],

      ]
    );

    $this->add_control(
      'readmore_icon',
      [
        'label' => esc_html__( 'Icon', 'recent-posts-elementor-addon' ),
        'type' => Controls_Manager::ICONS,
        'fa4compatibility' => 'icon',
        'condition' => [
          'show_readmore' => 'yes',
        ],

      ]
    );

    $this->add_control(
      'readmore_icon_align',
      [
        'label' => esc_html__( 'Icon Position', 'recent-posts-elementor-addon' ),
        'type' => Controls_Manager::SELECT,
        'default' => 'left',
        'options' => [
          'left' => esc_html__( 'Before', 'recent-posts-elementor-addon' ),
          'right' => esc_html__( 'After', 'recent-posts-elementor-addon' ),
        ],
        'condition' => [
          'show_readmore' => 'yes',
          'readmore_icon[value]!' => '',
        ],
      ]
    );

    $this->add_control(
      'top_meta',
      [
        'label' => esc_html__( 'Before Title', 'recent-posts-elementor-addon' ),
        'type' => Controls_Manager::SELECT2,
        'multiple' => true,
        'options' => [
          'date_meta' => esc_html__( 'Date', 'recent-posts-elementor-addon' ),
          'author_meta' => esc_html__( 'Author', 'recent-posts-elementor-addon' ),
          'category_meta' => esc_html__( 'Single Category', 'recent-posts-elementor-addon' ),
          'categories_meta' => esc_html__( 'Categories', 'recent-posts-elementor-addon' ),
        ],
        'default'=> ['category_meta'],
        'label_block' => true,
        'separator' =>'before'
      ]
    );

    $this->add_control(
      'top_meta_divider',
      [
        'label' => esc_html__( 'Divider', 'recent-posts-elementor-addon' ),
        'type' => Controls_Manager::CHOOSE,
        'default' => '',
        'options' => [
          '' => [
            'title' => esc_html__( 'None', 'recent-posts-elementor-addon' ),
            'icon' => 'eicon-ban',
          ],
          'text' => [
            'title' => esc_html__( 'Text', 'recent-posts-elementor-addon' ),
            'icon' => 'eicon-t-letter-bold',
          ],
          'icon' => [
            'title' => esc_html__( 'Icon', 'recent-posts-elementor-addon' ),
            'icon' => 'eicon-star',
          ],
        ],
        'toggle' => false,
        'condition' => [
          'top_meta!' => ''
        ],
      ]
    );

    $this->add_control(
      'top_divider_text',
      [
        'label' => esc_html__( 'Text', 'recent-posts-elementor-addon' ),
        'type' => Controls_Manager::TEXT,
        'condition' => [
          'top_meta_divider' => 'text',
          'top_meta!' => ''
        ],
        'default' => '-',
        'dynamic' => [
          'active' => false,
        ],
      ]
    );

    $this->add_control(
      'top_divider_icon',
      [
        'label' => esc_html__( 'Icon', 'recent-posts-elementor-addon' ),
        'type' => Controls_Manager::ICONS,
        'default' => [
          'value' => 'fas fa-caret-right',
          'library' => 'fa-solid',
        ],
        'condition' => [
          'top_meta!' => '',
          'top_meta_divider' => 'icon',
        ],
      ]
    );


    $this->add_control(
      'mid_meta',
      [
        'label' => esc_html__( 'After Title', 'recent-posts-elementor-addon' ),
        'type' => Controls_Manager::SELECT2,
        'multiple' => true,
        'options' => [
          'date_meta' => esc_html__( 'Date', 'recent-posts-elementor-addon' ),
          'author_meta' => esc_html__( 'Author', 'recent-posts-elementor-addon' ),
          'category_meta' => esc_html__( 'Single Category', 'recent-posts-elementor-addon' ),
          'categories_meta' => esc_html__( 'Categories', 'recent-posts-elementor-addon' ),
        ],
        'default'=> ['date_meta','author_meta'],
        'label_block' => true,
        'separator' =>'before'
      ]
    );

    $this->add_control(
      'mid_meta_divider',
      [
        'label' => esc_html__( 'Divider', 'recent-posts-elementor-addon' ),
        'type' => Controls_Manager::CHOOSE,
        'default' => '',
        'options' => [
          '' => [
            'title' => esc_html__( 'None', 'recent-posts-elementor-addon' ),
            'icon' => 'eicon-ban',
          ],
          'text' => [
            'title' => esc_html__( 'Text', 'recent-posts-elementor-addon' ),
            'icon' => 'eicon-t-letter-bold',
          ],
          'icon' => [
            'title' => esc_html__( 'Icon', 'recent-posts-elementor-addon' ),
            'icon' => 'eicon-star',
          ],
        ],
        'toggle' => false,
        'condition' => [
          'mid_meta!' => ''
        ],
      ]
    );

    $this->add_control(
      'mid_divider_text',
      [
        'label' => esc_html__( 'Text', 'recent-posts-elementor-addon' ),
        'type' => Controls_Manager::TEXT,
        'condition' => [
          'mid_meta_divider' => 'text',
          'mid_meta!' => ''
        ],
        'default' => '-',
        'dynamic' => [
          'active' => false,
        ],
      ]
    );

    $this->add_control(
      'mid_divider_icon',
      [
        'label' => esc_html__( 'Icon', 'recent-posts-elementor-addon' ),
        'type' => Controls_Manager::ICONS,
        'default' => [
          'value' => 'fas fa-caret-right',
          'library' => 'fa-solid',
        ],
        'condition' => [
          'mid_meta!' => '',
          'mid_meta_divider' => 'icon',
        ],
      ]
    );

    $this->add_control(
      'bottom_meta',
      [
        'label' => esc_html__( 'Bottom', 'recent-posts-elementor-addon' ),
        'type' => Controls_Manager::SELECT2,
        'multiple' => true,
        'options' => [
          'date_meta' => esc_html__( 'Date', 'recent-posts-elementor-addon' ),
          'author_meta' => esc_html__( 'Author', 'recent-posts-elementor-addon' ),
          'category_meta' => esc_html__( 'Single Category', 'recent-posts-elementor-addon' ),
          'categories_meta' => esc_html__( 'Categories', 'recent-posts-elementor-addon' ),
        ],
        'label_block' => true,
        'separator' =>'before'
      ]
    );

    $this->add_control(
      'bottom_meta_divider',
      [
        'label' => esc_html__( 'Divider', 'recent-posts-elementor-addon' ),
        'type' => Controls_Manager::CHOOSE,
        'default' => '',
        'options' => [
          '' => [
            'title' => esc_html__( 'None', 'recent-posts-elementor-addon' ),
            'icon' => 'eicon-ban',
          ],
          'text' => [
            'title' => esc_html__( 'Text', 'recent-posts-elementor-addon' ),
            'icon' => 'eicon-t-letter-bold',
          ],
          'icon' => [
            'title' => esc_html__( 'Icon', 'recent-posts-elementor-addon' ),
            'icon' => 'eicon-star',
          ],
        ],
        'toggle' => false,
        'condition' => [
          'bottom_meta!' => ''
        ],
      ]
    );

    $this->add_control(
      'bottom_divider_text',
      [
        'label' => esc_html__( 'Text', 'recent-posts-elementor-addon' ),
        'type' => Controls_Manager::TEXT,
        'condition' => [
          'bottom_meta_divider' => 'text',
          'bottom_meta!' => ''
        ],
        'default' => '-',
        'dynamic' => [
          'active' => false,
        ],
      ]
    );

    $this->add_control(
      'bottom_divider_icon',
      [
        'label' => esc_html__( 'Icon', 'recent-posts-elementor-addon' ),
        'type' => Controls_Manager::ICONS,
        'default' => [
          'value' => 'fas fa-caret-right',
          'library' => 'fa-solid',
        ],
        'condition' => [
          'bottom_meta!' => '',
          'bottom_meta_divider' => 'icon',
        ],
      ]
    );

    $this->end_controls_section();

    $this->start_controls_section(
      'section_data',
      [
        'label' => esc_html__( 'Query', 'recent-posts-elementor-addon' ),
      ]
    );

    $this->add_control(
      'filter_by',
      [
        'label' => esc_html__( 'Selection', 'recent-posts-elementor-addon' ),
        'type' => Controls_Manager::SELECT,
        'options' => [
          '' => esc_html__( 'Select All', 'recent-posts-elementor-addon' ),
          'category' => esc_html__( 'By Category', 'recent-posts-elementor-addon' ),
          'tag' => esc_html__( 'By Tag', 'recent-posts-elementor-addon' ),
          'ids' => esc_html__( 'By IDs', 'recent-posts-elementor-addon' ),
          'sticky' => esc_html__( 'Sticky Only', 'recent-posts-elementor-addon' )
        ],
        'default' => ''
      ]
    );



    $categories_options = array(
       ''=> esc_html__( 'Select Category', 'recent-posts-elementor-addon' ),
       'only'=> esc_html__( 'Same This Post', 'recent-posts-elementor-addon' )
     );

   $categories_args = array(
            'orderby' => 'name',
            'show_count' => 0,
            'pad_counts' => 0,
            'hierarchical' => 0,
    );


    $categories=get_categories($categories_args);

    if(count($categories)){

        foreach ( $categories as $category ) {
          $categories_options[$category->term_id] = $category->name;
        }

     }

    $this->add_control(
      'cat_ids',
      [
        'label' => esc_html__( 'By Category', 'recent-posts-elementor-addon' ),
        'type' => Controls_Manager::SELECT,
        'options' => $categories_options,
        'default' => '',
        'condition' => [
          'filter_by' => 'category'
        ],
      ]
    );


    $tags_options = array(
       ''=> esc_html__( 'Select Tag', 'recent-posts-elementor-addon' ),
     );

   $tags_args = array(
            'orderby' => 'name',
            'show_count' => 0,
            'pad_counts' => 0,
            'hierarchical' => 0,
    );


    $tags=get_tags($tags_args);

    if(count($tags)){

        foreach ( $tags as $term ) {
          $tags_options[$term->term_id] = $term->name;
        }

     }

    $this->add_control(
      'tags_ids',
      [
        'label' => esc_html__( 'By Tags', 'recent-posts-elementor-addon' ),
        'type' => Controls_Manager::SELECT,
        'options' => $tags_options,
        'default' => '',
        'condition' => [
          'filter_by' => 'tag'
        ],
      ]
    );

    $this->add_control(
      'posts_ids',
      [
        'label' => esc_html__( 'Post ID', 'recent-posts-elementor-addon' ),
        'type' => Controls_Manager::TEXT,
        'default' => '',
        'description' => esc_html__( 'Type post IDs. Seperated by comma', 'recent-posts-elementor-addon' ),
        'label_block' => true,
        'condition' => [
          'filter_by' => 'ids'
        ],
      ]
    );

    $this->add_control(
      'posts_per_page',
      [
        'label' => esc_html__( 'Post Count', 'recent-posts-elementor-addon' ),
        'type' => Controls_Manager::NUMBER,
        'min' => 1,
        'max' => 100,
        'step' => 1,
        'default'=> 4
      ]
    );

    $this->add_control(
      'posts_offset',
      [
        'label' => esc_html__( 'Offset', 'recent-posts-elementor-addon' ),
        'type' => Controls_Manager::NUMBER,
        'min' => 1,
        'max' => 1000,
        'step' => 1,
        'default'=> 0
      ]
    );

    $this->add_control(
      'is_featured_image',
      [
        'label' => esc_html__( 'Featured Image Only', 'recent-posts-elementor-addon' ),
        'type' => Controls_Manager::SWITCHER,
        'label_off' => esc_html__( 'No', 'recent-posts-elementor-addon' ),
        'label_on' => esc_html__( 'Yes', 'recent-posts-elementor-addon' ),
        'default' => '',
      ]
    );

    $this->add_control(
      'skip_previous',
      [
        'label' => esc_html__( 'Exclude Previous Query', 'recent-posts-elementor-addon' ),
        'type' => Controls_Manager::SWITCHER,
        'label_off' => esc_html__( 'No', 'recent-posts-elementor-addon' ),
        'label_on' => esc_html__( 'Yes', 'recent-posts-elementor-addon' ),
        'default' => '',
        'description' => esc_html__( 'Exclude post from previous module query', 'recent-posts-elementor-addon' ),
      ]
    );

    $this->add_control(
      'single_exclude',
      [
        'label' => esc_html__( 'Exclude Current Post?', 'recent-posts-elementor-addon' ),
        'type' => Controls_Manager::SWITCHER,
        'label_off' => esc_html__( 'No', 'recent-posts-elementor-addon' ),
        'label_on' => esc_html__( 'Yes', 'recent-posts-elementor-addon' ),
        'default' => '',
        'description' => esc_html__( 'Select yes if current active post not showing in the listing. This option used on single post.', 'recent-posts-elementor-addon' ),
      ]
    );

    $this->add_control(
      'source_orderby',
      [
        'label' => esc_html__( 'Order By', 'recent-posts-elementor-addon' ),
        'type' => Controls_Manager::SELECT,
        'options' => [
          'latest' => esc_html__( 'Latest Post', 'recent-posts-elementor-addon' ),
          'comment' => esc_html__( 'Most Comment', 'recent-posts-elementor-addon' )
        ],
        'default' => 'latest'
      ]
    );

    $this->end_controls_section();

/*
 * style params
 */

    $this->start_controls_section(
      'post_grid_style',
      [
        'label' => esc_html__( 'Grid Style', 'recent-posts-elementor-addon' ),
        'tab'   => Controls_Manager::TAB_STYLE,
      ]
    );    

    $this->add_responsive_control(
      'post_grid_height',
      [
        'label' => esc_html__( 'Min Height', 'recent-posts-elementor-addon' ),
        'type' => Controls_Manager::SLIDER,
        'range' => [
          'px' => [
            'max' => 1000,
            'step' => 5,
            'min' => 50,
          ],
          'vh' => [
            'max' => 100,
            'step' => 1,
            'min' => 10,
          ],

        ],  
        'default'=>['size'=>'','unit'=>'px'],
        'size_units' => [ 'px' ,'vh' ],
        'selectors' => [
          '{{WRAPPER}} article' => 'min-height: {{SIZE}}{{UNIT}};',
        ],
      ]
    );

    $this->add_responsive_control(
      'post_grid_gutter',
      [
        'label' => esc_html__( 'Spacing', 'recent-posts-elementor-addon' ),
        'type' => Controls_Manager::SLIDER,
        'range' => [
         'px' => [
            'max' => 1000,
          ],
        ],  
        'default'=>['size'=>'30','unit'=>'px'],
        'size_units' => [ 'px' ],
        'selectors' => [
          '{{WRAPPER}} .gum_posts .post-list' => 'padding-bottom: {{SIZE}}{{UNIT}};',
        ]
       ]
    );

    $this->add_responsive_control(
      'post_grid_padding',
      [
          'label' => esc_html__( 'Padding', 'recent-posts-elementor-addon' ),
          'type' => Controls_Manager::DIMENSIONS,
          'size_units' => [ 'px', '%', 'em' ],
          'selectors' => [
              '{{WRAPPER}}  article' => 'padding: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
          ],
      ]
    );

    $this->add_control(
      'post_grid_radius',
      [
        'label' => esc_html__( 'Border Radius', 'recent-posts-elementor-addon' ),
        'type' => Controls_Manager::DIMENSIONS,
        'size_units' => [ 'px', '%' ],
        'selectors' => [
          '{{WRAPPER}} article' => 'border-radius: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
        ],
      ]
    );
    
    $this->add_group_control(
     Group_Control_Border::get_type(),
      [
        'name' => 'post_grid_border',
        'selector' => '{{WRAPPER}}  article',
      ]
    );

    $this->add_control(
      'post_grid_bdhover',
      [
        'label' => esc_html__( 'Hover Color', 'recent-posts-elementor-addon' ),
        'type' =>  Controls_Manager::COLOR,
        'default' => '',
        'selectors' => [
          '{{WRAPPER}} article:hover' => 'border-color: {{VALUE}};',
        ],
        'condition' => [
          'post_grid_border_border!' => ''
        ],
      ]
    );

    $this->add_control(
      'post_grid_bgcolor',
      [
        'label' => esc_html__( 'Background', 'elementor' ),
        'type' =>  Controls_Manager::COLOR,
        'default' => '',
        'selectors' => [
          '{{WRAPPER}} article' => 'background-color: {{VALUE}};',
        ]
      ]
    );

    $this->add_control(
      'post_grid_bghover',
      [
        'label' => esc_html__( 'Hover Background', 'recent-posts-elementor-addon' ),
        'type' =>  Controls_Manager::COLOR,
        'default' => '',
        'selectors' => [
          '{{WRAPPER}} article:hover' => 'background-color: {{VALUE}};',
        ]
      ]
    );


    $this->add_responsive_control(
      'list_animation',
      [
        'label' => esc_html__( 'Entrance Animation', 'elementor' ),
        'type' => Controls_Manager::ANIMATION,
      ]
    );

    $this->add_control(
      'list_animation_duration',
      [
        'label' => esc_html__( 'Animation Duration', 'elementor' ),
        'type' => Controls_Manager::SELECT,
        'default' => '',
        'options' => [
          'slow' => esc_html__( 'Slow', 'elementor' ),
          '' => esc_html__( 'Normal', 'elementor' ),
          'fast' => esc_html__( 'Fast', 'elementor' ),
        ],
        'condition' => [
          'list_animation!' => '',
        ],
      ]
    );

    $this->add_control(
      'list_animation_delay',
      [
        'label' => esc_html__( 'Animation Delay', 'elementor' ) . ' (ms)',
        'type' => Controls_Manager::NUMBER,
        'default' => '',
        'min' => 0,
        'step' => 100,
        'condition' => [
          'list_animation!' => '',
        ],
        'render_type' => 'none',
      ]
    );

    $this->end_controls_section();

    $this->start_controls_section(
      'post_list_image',
      [
        'label' => esc_html__( 'Image Box', 'recent-posts-elementor-addon' ),
        'tab'   => Controls_Manager::TAB_STYLE,
        'condition' => [
          'show_image!' => ''
        ],
      ]
    );    

    $this->add_responsive_control(
      'post_image_height',
      [
        'label' => esc_html__( 'Height', 'recent-posts-elementor-addon' ),
        'type' => Controls_Manager::SLIDER,
        'range' => [
          'px' => [
            'max' => 1000,
            'step' => 5,
            'min' => 50,
          ],
          'vh' => [
            'max' => 100,
            'step' => 1,
            'min' => 10,
          ],

        ],  
        'default'=>['size'=>'','unit'=>'px'],
        'size_units' => [ 'px' ,'vh' ],
        'selectors' => [
          '{{WRAPPER}} article .blog-image' => 'height: {{SIZE}}{{UNIT}};',
          '{{WRAPPER}} article .blog-image img' => 'height: {{SIZE}}{{UNIT}};',
        ],
        'condition' => [
          'image_position' => 'top'
        ],
      ]
    );

    $this->add_responsive_control(
      'post_image_mobileheight',
      [
        'label' => esc_html__( 'Height', 'recent-posts-elementor-addon' ),
        'type' => Controls_Manager::SLIDER,
        'range' => [
          'px' => [
            'max' => 1000,
            'step' => 5,
            'min' => 50,
          ],
          'vh' => [
            'max' => 100,
            'step' => 1,
            'min' => 10,
          ],

        ],  
        'default'=>['size'=>'','unit'=>'px'],
        'devices' => ['mobile'],
        'size_units' => [ 'px' ,'vh' ],
        'selectors' => [
          '{{WRAPPER}} article .blog-image' => 'height: {{SIZE}}{{UNIT}};',
          '{{WRAPPER}} article .blog-image img' => 'height: {{SIZE}}{{UNIT}};',
        ],
        'condition' => [
          'image_totop' => 'yes',
          'image_position!' => 'top',
        ],
      ]
    );
  
    $this->add_responsive_control(
      'post_image_width',
      [
        'label' => esc_html__( 'Width (%)', 'recent-posts-elementor-addon' ),
        'type' => Controls_Manager::SLIDER,
        'range' => [
          '%' => [
            'max' => 80,
            'step' => 5,
            'min' => 20,
          ],

        ],  
        'default'=>['size'=>50,'unit'=>'%'],
        'size_units' => [ '%' ],
        'selectors' => [
          '{{WRAPPER}} article .post-top' => 'width: {{SIZE}}%;',
          '{{WRAPPER}} article .post-top + .post-content' => 'width: calc( 100% - {{SIZE}}% );',
        ],
        'condition' => [
          'image_position!' => 'top'
        ],
      ]
    );

    $this->add_responsive_control(
      'post_image_margin',
      [
          'label' => esc_html__( 'Margin', 'recent-posts-elementor-addon' ),
          'type' => Controls_Manager::DIMENSIONS,
          'size_units' => [ 'px', '%', 'em' ],
          'selectors' => [
              '{{WRAPPER}} article .post-top' => 'margin: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
          ],
      ]
    );

    $this->add_control(
      'post_image_radius',
      [
        'label' => esc_html__( 'Border Radius', 'recent-posts-elementor-addon' ),
        'type' => Controls_Manager::DIMENSIONS,
        'size_units' => [ 'px', '%' ],
        'selectors' => [
          '{{WRAPPER}} article .blog-image' => 'border-radius: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
        ],
      ]
    );

    $this->add_group_control(
     Group_Control_Border::get_type(),
      [
        'name' => 'post_image_border',
        'selector' => '{{WRAPPER}} article .blog-image',
      ]
    );


    $this->add_control(
      'post_image_bdhover',
      [
        'label' => esc_html__( 'Hover Color', 'recent-posts-elementor-addon' ),
        'type' =>  Controls_Manager::COLOR,
        'default' => '',
        'selectors' => [
          '{{WRAPPER}} article:hover .blog-image' => 'border-color: {{VALUE}};',
        ],
        'condition' => [
          'post_image_border_border!' => ''
        ],
      ]
    );


    $this->end_controls_section();

    $this->start_controls_section(
      'post_title_style',
      [
        'label' => esc_html__( 'Content Box', 'recent-posts-elementor-addon' ),
        'tab'   => Controls_Manager::TAB_STYLE,
      ]
    );    

    $this->add_responsive_control(
      'grid_content_align',
      [
        'label' => esc_html__( 'Content Align', 'recent-posts-elementor-addon' ),
        'type' => Controls_Manager::CHOOSE,
        'options' => [
              'left' => [
                'title' => esc_html__( 'Left', 'recent-posts-elementor-addon' ),
                'icon' => 'eicon-text-align-left',
              ],
              'center' => [
                'title' => esc_html__( 'Center', 'recent-posts-elementor-addon' ),
                'icon' => 'eicon-text-align-center',
              ],
              'right' => [
                'title' => esc_html__( 'Right', 'recent-posts-elementor-addon' ),
                'icon' => 'eicon-text-align-right',
              ],
              'justify' => [
                'title' => esc_html__( 'Justify', 'recent-posts-elementor-addon' ),
                'icon' => 'eicon-text-align-justify',
              ]
        ],
        'default' => '',
        'selectors' => [
          '{{WRAPPER}} article .post-content' => 'text-align: {{VALUE}};',
        ],
      ]
    );

    $this->add_responsive_control(
      'grid_content_padding',
      [
          'label' => esc_html__( 'Padding', 'recent-posts-elementor-addon' ),
          'type' => Controls_Manager::DIMENSIONS,
          'size_units' => [ 'px', '%', 'em' ],
          'selectors' => [
              '{{WRAPPER}} article .post-content' => 'padding: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}} !important;',
          ],
      ]
    );

    $this->add_control(
      'grid_content_radius',
      [
        'label' => esc_html__( 'Border Radius', 'recent-posts-elementor-addon' ),
        'type' => Controls_Manager::DIMENSIONS,
        'size_units' => [ 'px', '%' ],
        'selectors' => [
          '{{WRAPPER}} article .post-content' => 'border-radius: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
        ],
      ]
    );

    $this->add_group_control(
     Group_Control_Border::get_type(),
      [
        'name' => 'grid_content_border',
        'selector' => '{{WRAPPER}} article .post-content',
      ]
    );

    $this->add_control(
      'grid_content_bdhover',
      [
        'label' => esc_html__( 'Hover Color', 'recent-posts-elementor-addon' ),
        'type' =>  Controls_Manager::COLOR,
        'default' => '',
        'selectors' => [
          '{{WRAPPER}} article:hover .post-content' => 'border-color: {{VALUE}};',
        ],
        'condition' => [
          'grid_content_border_border!' => ''
        ],
      ]
    );

    $this->add_control(
      'grid_content_bgcolor',
      [
        'label' => esc_html__( 'Background', 'elementor' ),
        'type' =>  Controls_Manager::COLOR,
        'default' => '',
        'selectors' => [
          '{{WRAPPER}} article .post-content' => 'background-color: {{VALUE}};',
        ]
      ]
    );

    $this->add_control(
      'grid_content_bghover',
      [
        'label' => esc_html__( 'Hover Background', 'recent-posts-elementor-addon' ),
        'type' =>  Controls_Manager::COLOR,
        'default' => '',
        'selectors' => [
          '{{WRAPPER}} article:hover .post-content' => 'background-color: {{VALUE}};',
        ]
      ]
    );

    $this->end_controls_section();

    $this->start_controls_section(
      'post_meta_style',
      [
        'label' => esc_html__( 'Meta - Before Title', 'recent-posts-elementor-addon' ),
        'tab'   => Controls_Manager::TAB_STYLE,
        'condition' => [
          'top_meta!' =>''
        ],
      ]
    );  

    $this->add_responsive_control(
      'post_meta_margin',
      [
        'label' => esc_html__( 'Margin', 'recent-posts-elementor-addon' ),
        'type' => Controls_Manager::DIMENSIONS,
        'size_units' => [ 'px', '%', 'em' ],
        'selectors' => [
            '{{WRAPPER}} .meta-position-top' => 'margin: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
        ],
      ]
    );

    $this->add_responsive_control(
      'post_meta_gap',
      [
        'label' => esc_html__( 'Vertical Spacing', 'recent-posts-elementor-addon' ),
        'type' => Controls_Manager::SLIDER,
        'range' => [
          'px' => [
            'max' => 100,
            'step' => 1,
          ],
        ],  
        'default'=>['size'=>'','unit'=>'px'],
        'size_units' => [ 'px'],
        'selectors' => [
          '{{WRAPPER}} .meta-position-top li' => 'margin-bottom: {{SIZE}}{{UNIT}};',
        ],
      ]
    );

    $this->add_group_control(
      Group_Control_Typography::get_type(),
      [
        'name' => 'typography_meta_title',
        'selector' => '{{WRAPPER}} .meta-position-top .list-meta a,{{WRAPPER}} .meta-position-top .list-meta .meta-text',
      ]
    );


   $this->start_controls_tabs( 'meta_title_tabs', [] );

   $this->start_controls_tab(
       'meta_title_normal',
       [
           'label' =>esc_html__( 'Normal', 'elementor' ),
       ]
   );

    $this->add_control(
      'meta_title_color',
      [
        'label' => esc_html__( 'Color', 'elementor' ),
        'type' =>  Controls_Manager::COLOR,
        'default' => '',
        'selectors' => [
          '{{WRAPPER}} .meta-position-top .list-meta a,{{WRAPPER}} .meta-position-top .list-meta .meta-text' => 'color: {{VALUE}};',
        ]
      ]
    );

    $this->add_control(
      'meta_list_bgcolor',
      [
        'label' => esc_html__( 'Background', 'elementor' ),
        'type' =>  Controls_Manager::COLOR,
        'default' => '',
        'selectors' => [
          '{{WRAPPER}} .meta-position-top .list-meta' => 'background-color: {{VALUE}};',
        ]
      ]
    );

   $this->end_controls_tab();

   $this->start_controls_tab(
       'meta_title_hover',
       [
           'label' =>esc_html__( 'Hover', 'elementor' ),
       ]
   );

    $this->add_control(
      'meta_title_hcolor',
      [
        'label' => esc_html__( 'Color', 'elementor' ),
        'type' =>  Controls_Manager::COLOR,
        'default' => '',
        'selectors' => [
          '{{WRAPPER}} .meta-position-top .list-meta:hover a,{{WRAPPER}} .meta-position-top .list-meta:hover .meta-text' => 'color: {{VALUE}};',
        ]
      ]
    );

    $this->add_control(
      'meta_list_bghover',
      [
        'label' => esc_html__( 'Background', 'elementor' ),
        'type' =>  Controls_Manager::COLOR,
        'default' => '',
        'selectors' => [
          '{{WRAPPER}} .meta-position-top .list-meta:hover' => 'background-color: {{VALUE}};',
        ]
      ]
    );

    $this->add_control(
      'meta_list_bdhover',
      [
        'label' => esc_html__( 'Border Color', 'recent-posts-elementor-addon' ),
        'type' =>  Controls_Manager::COLOR,
        'default' => '',
        'selectors' => [
          '{{WRAPPER}} .meta-position-top .list-meta:hover' => 'border-color: {{VALUE}};',
        ],
        'condition' => [
          'meta_list_border_border!' => ''
        ],
      ]
    );

   $this->end_controls_tab();
   $this->end_controls_tabs();


    $this->add_responsive_control(
        'meta_list_padding',
        [
            'label' => esc_html__( 'Padding', 'recent-posts-elementor-addon' ),
            'type' => Controls_Manager::DIMENSIONS,
            'size_units' => [ 'px', '%', 'em' ],
            'selectors' => [
                '{{WRAPPER}} .meta-position-top .list-meta' => 'padding: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
            ],
        ]
    );

    $this->add_control(
      'meta_list_radius',
      [
        'label' => esc_html__( 'Border Radius', 'recent-posts-elementor-addon' ),
        'type' => Controls_Manager::DIMENSIONS,
        'size_units' => [ 'px', '%' ],
        'selectors' => [
          '{{WRAPPER}} .meta-position-top .list-meta' => 'border-radius: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
        ],
      ]
    );

    $this->add_group_control(
     Group_Control_Border::get_type(),
      [
        'name' => 'meta_list_border',
        'selector' => '{{WRAPPER}} .meta-position-top .list-meta',
      ]
    );

    $this->add_control(
      'divider_heading',
      [
        'label' => esc_html__( 'Divider', 'recent-posts-elementor-addon' ),
        'type' => Controls_Manager::HEADING,
        'separator' => 'before',
      ]
    );

    $this->add_responsive_control(
      'divider_size',
      [
        'label' => esc_html__( 'Size', 'recent-posts-elementor-addon' ),
        'type' => Controls_Manager::SLIDER,
        'range' => [
          'em' => [
            'max' => 10,
            'step'=>1
          ],
          'px' => [
            'max' => 200,
            'step'=>1
          ],
        ],  
        'default'=>['size'=>1,'unit'=>'em'],
        'size_units' => [ 'px', 'em' ],
        'selectors' => [
          '{{WRAPPER}} .meta-position-top .meta-divider span' => 'font-size: {{SIZE}}{{UNIT}};',
        ],
        'condition' => [
          'top_meta_divider!' => ''
        ],
      ]
    );

    $this->add_control(
      'divider_color',
      [
        'label' => esc_html__( 'Color', 'elementor' ),
        'type' =>  Controls_Manager::COLOR,
        'default' => '',
        'selectors' => [
          '{{WRAPPER}} .meta-position-top .meta-divider span' => 'color: {{VALUE}};',
        ],
        'condition' => [
          'top_meta_divider!' => ''
        ],
      ]
    );

    $this->add_responsive_control(
        'divider_padding',
        [
            'label' => esc_html__( 'Padding', 'recent-posts-elementor-addon' ),
            'type' => Controls_Manager::DIMENSIONS,
            'size_units' => [ 'px', 'em' ],
            'default' => [],
            'selectors' => [
                '{{WRAPPER}} .meta-position-top .meta-divider' => 'padding: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
            ],
        ]
    );

    $this->end_controls_section();

    $this->start_controls_section(
      'post_title_styles',
      [
        'label' => esc_html__( 'Post Title', 'recent-posts-elementor-addon' ),
        'tab'   => Controls_Manager::TAB_STYLE,
      ]
    );  

    $this->add_responsive_control(
      'post_title_margin',
      [
          'label' => esc_html__( 'Margin', 'recent-posts-elementor-addon' ),
          'type' => Controls_Manager::DIMENSIONS,
          'size_units' => [ 'px', '%', 'em' ],
          'selectors' => [
              '{{WRAPPER}} article .post-title' => 'margin: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
          ],
      ]
    );

    $this->add_group_control(
      Group_Control_Typography::get_type(),
      [
        'name' => 'typography_post_title',
        'selector' => '{{WRAPPER}} .post-title',
      ]
    );

    $this->add_control(
      'post_title_color',
      [
        'label' => esc_html__( 'Color', 'elementor' ),
        'type' =>  Controls_Manager::COLOR,
        'default' => '',
        'selectors' => [
          '{{WRAPPER}} .post-title' => 'color: {{VALUE}};',
        ]
      ]
    );

    $this->add_control(
      'post_title_hcolor',
      [
        'label' => esc_html__( 'Hover Color', 'recent-posts-elementor-addon' ),
        'type' =>  Controls_Manager::COLOR,
        'default' => '',
        'selectors' => [
          '{{WRAPPER}} article .post-title:hover,{{WRAPPER}} .post-title:focus' => 'color: {{VALUE}};',
        ]
      ]
    );

    $this->end_controls_section();

    $this->start_controls_section(
      'post_midmeta_style',
      [
        'label' => esc_html__( 'Meta - After Title', 'recent-posts-elementor-addon' ),
        'tab'   => Controls_Manager::TAB_STYLE,
        'condition' => [
          'mid_meta!' =>''
        ],
      ]
    );  

    $this->add_responsive_control(
      'post_midmeta_margin',
      [
        'label' => esc_html__( 'Margin', 'recent-posts-elementor-addon' ),
        'type' => Controls_Manager::DIMENSIONS,
        'size_units' => [ 'px', '%', 'em' ],
        'selectors' => [
            '{{WRAPPER}} .meta-position-mid' => 'margin: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
        ],
      ]
    );


    $this->add_responsive_control(
      'post_midmeta_gap',
      [
        'label' => esc_html__( 'Vertical Spacing', 'recent-posts-elementor-addon' ),
        'type' => Controls_Manager::SLIDER,
        'range' => [
          'px' => [
            'max' => 100,
            'step' => 1,
          ],
        ],  
        'default'=>['size'=>'','unit'=>'px'],
        'size_units' => [ 'px'],
        'selectors' => [
          '{{WRAPPER}} .meta-position-mid li' => 'margin-bottom: {{SIZE}}{{UNIT}};',
        ],
      ]
    );

    $this->add_group_control(
      Group_Control_Typography::get_type(),
      [
        'name' => 'typography_midmeta_title',
        'selector' => '{{WRAPPER}} .meta-position-mid .list-meta a,{{WRAPPER}} .meta-position-mid .list-meta .meta-text',
      ]
    );

   $this->start_controls_tabs( 'midmeta_title_tabs', [] );

   $this->start_controls_tab(
       'midmeta_title_normal',
       [
           'label' =>esc_html__( 'Normal', 'elementor' ),
       ]
   );

    $this->add_control(
      'midmeta_title_color',
      [
        'label' => esc_html__( 'Color', 'elementor' ),
        'type' =>  Controls_Manager::COLOR,
        'default' => '',
        'selectors' => [
          '{{WRAPPER}} .meta-position-mid .list-meta a,{{WRAPPER}} .meta-position-mid .list-meta .meta-text' => 'color: {{VALUE}};',
        ]
      ]
    );


    $this->add_control(
      'midmeta_list_bgcolor',
      [
        'label' => esc_html__( 'Background', 'elementor' ),
        'type' =>  Controls_Manager::COLOR,
        'default' => '',
        'selectors' => [
          '{{WRAPPER}} .meta-position-mid .list-meta' => 'background-color: {{VALUE}};',
        ]
      ]
    );

   $this->end_controls_tab();

   $this->start_controls_tab(
       'midmeta_title_hover',
       [
           'label' =>esc_html__( 'Hover', 'elementor' ),
       ]
   );

    $this->add_control(
      'midmeta_title_hcolor',
      [
        'label' => esc_html__( 'Color', 'elementor' ),
        'type' =>  Controls_Manager::COLOR,
        'default' => '',
        'selectors' => [
          '{{WRAPPER}} .meta-position-mid .list-meta:hover a,{{WRAPPER}} .meta-position-mid .list-meta:hover .meta-text' => 'color: {{VALUE}};',
        ]
      ]
    );

    $this->add_control(
      'midmeta_list_bghover',
      [
        'label' => esc_html__( 'Background', 'elementor' ),
        'type' =>  Controls_Manager::COLOR,
        'default' => '',
        'selectors' => [
          '{{WRAPPER}} .meta-position-mid .list-meta:hover' => 'background-color: {{VALUE}};',
        ]
      ]
    );

    $this->add_control(
      'midmeta_list_bdhover',
      [
        'label' => esc_html__( 'Border Color', 'recent-posts-elementor-addon' ),
        'type' =>  Controls_Manager::COLOR,
        'default' => '',
        'selectors' => [
          '{{WRAPPER}} .meta-position-mid .list-meta:hover' => 'border-color: {{VALUE}};',
        ],
        'condition' => [
          'midmeta_list_border_border!' => ''
        ],
      ]
    );


   $this->end_controls_tab();
   $this->end_controls_tabs();

    $this->add_responsive_control(
        'midmeta_list_padding',
        [
            'label' => esc_html__( 'Padding', 'recent-posts-elementor-addon' ),
            'type' => Controls_Manager::DIMENSIONS,
            'size_units' => [ 'px', '%', 'em' ],
            'selectors' => [
                '{{WRAPPER}} .meta-position-mid .list-meta' => 'padding: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
            ],
        ]
    );

    $this->add_control(
      'midmeta_list_radius',
      [
        'label' => esc_html__( 'Border Radius', 'recent-posts-elementor-addon' ),
        'type' => Controls_Manager::DIMENSIONS,
        'size_units' => [ 'px', '%' ],
        'selectors' => [
          '{{WRAPPER}} .meta-position-mid .list-meta' => 'border-radius: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
        ],
      ]
    );

    $this->add_group_control(
     Group_Control_Border::get_type(),
      [
        'name' => 'midmeta_list_border',
        'selector' => '{{WRAPPER}} .meta-position-mid .list-meta',
      ]
    );

    $this->add_control(
      'mid_divider_heading',
      [
        'label' => esc_html__( 'Divider', 'recent-posts-elementor-addon' ),
        'type' => Controls_Manager::HEADING,
        'separator' => 'before',
      ]
    );

    $this->add_responsive_control(
      'mid_divider_size',
      [
        'label' => esc_html__( 'Size', 'recent-posts-elementor-addon' ),
        'type' => Controls_Manager::SLIDER,
        'range' => [
          'em' => [
            'max' => 10,
            'step'=>1
          ],
          'px' => [
            'max' => 200,
            'step'=>1
          ],
        ],  
        'default'=>['size'=>1,'unit'=>'em'],
        'size_units' => [ 'px', 'em' ],
        'selectors' => [
          '{{WRAPPER}} .meta-position-mid .meta-divider span' => 'font-size: {{SIZE}}{{UNIT}};',
        ],
        'condition' => [
          'mid_meta_divider!' => ''
        ],
      ]
    );

    $this->add_control(
      'mid_divider_color',
      [
        'label' => esc_html__( 'Color', 'elementor' ),
        'type' =>  Controls_Manager::COLOR,
        'default' => '',
        'selectors' => [
          '{{WRAPPER}} .meta-position-mid .meta-divider span' => 'color: {{VALUE}};',
        ],
        'condition' => [
          'mid_meta_divider!' => ''
        ],
      ]
    );

    $this->add_responsive_control(
        'mid_divider_padding',
        [
            'label' => esc_html__( 'Padding', 'recent-posts-elementor-addon' ),
            'type' => Controls_Manager::DIMENSIONS,
            'size_units' => [ 'px', 'em' ],
            'default' => [],
            'selectors' => [
                '{{WRAPPER}} .meta-position-mid .meta-divider' => 'padding: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
            ],
        ]
    );

    $this->end_controls_section();


    $this->start_controls_section(
      'post_content_styles',
      [
        'label' => esc_html__( 'Post Excerpt', 'recent-posts-elementor-addon' ),
        'tab'   => Controls_Manager::TAB_STYLE,
        'condition' => [
          'show_excerpt' => 'yes',
        ],
      ]
    );  

    $this->add_responsive_control(
      'post_content_margin',
      [
        'label' => esc_html__( 'Margin', 'recent-posts-elementor-addon' ),
        'type' => Controls_Manager::DIMENSIONS,
        'size_units' => [ 'px', '%', 'em' ],
        'selectors' => [
            '{{WRAPPER}} article .content-excerpt' => 'margin: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
        ],
        'condition' => [
          'show_excerpt' => 'yes',
        ],
      ]
    );

    $this->add_group_control(
      Group_Control_Typography::get_type(),
      [
        'name' => 'typography_post_content',
        'selector' => '{{WRAPPER}} article .content-excerpt',
        'condition' => [
          'show_excerpt' => 'yes',
        ],
      ]
    );

    $this->add_control(
      'post_content_color',
      [
        'label' => esc_html__( 'Color', 'elementor' ),
        'type' =>  Controls_Manager::COLOR,
        'default' => '',
        'selectors' => [
          '{{WRAPPER}} article .content-excerpt' => 'color: {{VALUE}};',
        ],
        'condition' => [
          'show_excerpt' => 'yes',
        ],
      ]
    );


    $this->add_control(
      'post_content_hcolor',
      [
        'label' => esc_html__( 'Hover Color', 'recent-posts-elementor-addon' ),
        'type' =>  Controls_Manager::COLOR,
        'default' => '',
        'selectors' => [
          '{{WRAPPER}} article:hover .content-excerpt,{{WRAPPER}} article:focus .content-excerpt' => 'color: {{VALUE}};',
        ],
        'condition' => [
          'show_excerpt' => 'yes',
        ],
      ]
    );

    $this->end_controls_section();


    $this->start_controls_section(
      'post_bottommeta_style',
      [
        'label' => esc_html__( 'Meta - After Content', 'recent-posts-elementor-addon' ),
        'tab'   => Controls_Manager::TAB_STYLE,
        'condition' => [
          'bottom_meta!' =>''
        ],
      ]
    );  

    $this->add_responsive_control(
      'post_bottommeta_margin',
      [
        'label' => esc_html__( 'Margin', 'recent-posts-elementor-addon' ),
        'type' => Controls_Manager::DIMENSIONS,
        'size_units' => [ 'px', '%', 'em' ],
        'selectors' => [
            '{{WRAPPER}} .meta-position-bottom' => 'margin: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
        ],
      ]
    );


    $this->add_responsive_control(
      'post_bottommeta_gap',
      [
        'label' => esc_html__( 'Vertical Spacing', 'recent-posts-elementor-addon' ),
        'type' => Controls_Manager::SLIDER,
        'range' => [
          'px' => [
            'max' => 100,
            'step' => 1,
          ],
        ],  
        'default'=>['size'=>'','unit'=>'px'],
        'size_units' => [ 'px'],
        'selectors' => [
          '{{WRAPPER}} .meta-position-bottom li' => 'margin-bottom: {{SIZE}}{{UNIT}};',
        ],
      ]
    );
    $this->add_group_control(
      Group_Control_Typography::get_type(),
      [
        'name' => 'typography_bottommeta_title',
        'selector' => '{{WRAPPER}} .meta-position-bottom .list-meta a,{{WRAPPER}} .meta-position-bottom .list-meta .meta-text',
      ]
    );

   $this->start_controls_tabs( 'bottommeta_title_tabs', [] );

   $this->start_controls_tab(
       'bottommeta_title_normal',
       [
           'label' =>esc_html__( 'Normal', 'elementor' ),
       ]
   );

    $this->add_control(
      'bottommeta_title_color',
      [
        'label' => esc_html__( 'Color', 'elementor' ),
        'type' =>  Controls_Manager::COLOR,
        'default' => '',
        'selectors' => [
          '{{WRAPPER}} .meta-position-bottom .list-meta a,{{WRAPPER}} .meta-position-bottom .list-meta .meta-text' => 'color: {{VALUE}};',
        ]
      ]
    );

    $this->add_control(
      'bottommeta_list_bgcolor',
      [
        'label' => esc_html__( 'Background', 'elementor' ),
        'type' =>  Controls_Manager::COLOR,
        'default' => '',
        'selectors' => [
          '{{WRAPPER}} .meta-position-bottom .list-meta' => 'background-color: {{VALUE}};',
        ]
      ]
    );

   $this->end_controls_tab();

   $this->start_controls_tab(
       'bottommeta_title_hover',
       [
           'label' =>esc_html__( 'Hover', 'elementor' ),
       ]
   );

    $this->add_control(
      'bottommeta_title_hcolor',
      [
        'label' => esc_html__( 'Color', 'elementor' ),
        'type' =>  Controls_Manager::COLOR,
        'default' => '',
        'selectors' => [
          '{{WRAPPER}} .meta-position-bottom .list-meta:hover a,{{WRAPPER}} .meta-position-bottom .list-meta:hover .meta-text' => 'color: {{VALUE}};',
        ]
      ]
    );

    $this->add_control(
      'bottommeta_list_bghover',
      [
        'label' => esc_html__( 'Background', 'elementor' ),
        'type' =>  Controls_Manager::COLOR,
        'default' => '',
        'selectors' => [
          '{{WRAPPER}} .meta-position-bottom .list-meta:hover' => 'background-color: {{VALUE}};',
        ]
      ]
    );

    $this->add_control(
      'bottommeta_list_bdhover',
      [
        'label' => esc_html__( 'Border Color', 'recent-posts-elementor-addon' ),
        'type' =>  Controls_Manager::COLOR,
        'default' => '',
        'selectors' => [
          '{{WRAPPER}} .meta-position-bottom .list-meta:hover' => 'border-color: {{VALUE}};',
        ],
        'condition' => [
          'bottommeta_list_border_border!' => ''
        ],
      ]
    );

   $this->end_controls_tab();
   $this->end_controls_tabs();

    $this->add_responsive_control(
        'bottommeta_list_padding',
        [
            'label' => esc_html__( 'Padding', 'recent-posts-elementor-addon' ),
            'type' => Controls_Manager::DIMENSIONS,
            'size_units' => [ 'px', '%', 'em' ],
            'selectors' => [
                '{{WRAPPER}} .meta-position-bottom .list-meta' => 'padding: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
            ],
        ]
    );

    $this->add_control(
      'bottommeta_list_radius',
      [
        'label' => esc_html__( 'Border Radius', 'recent-posts-elementor-addon' ),
        'type' => Controls_Manager::DIMENSIONS,
        'size_units' => [ 'px', '%' ],
        'selectors' => [
          '{{WRAPPER}} .meta-position-bottom .list-meta' => 'border-radius: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
        ],
      ]
    );

    $this->add_group_control(
     Group_Control_Border::get_type(),
      [
        'name' => 'bottommeta_list_border',
        'selector' => '{{WRAPPER}} .meta-position-bottom .list-meta',
      ]
    );

    $this->add_control(
      'bottom_divider_heading',
      [
        'label' => esc_html__( 'Divider', 'recent-posts-elementor-addon' ),
        'type' => Controls_Manager::HEADING,
        'separator' => 'before',
      ]
    );

    $this->add_responsive_control(
      'bottom_divider_size',
      [
        'label' => esc_html__( 'Size', 'recent-posts-elementor-addon' ),
        'type' => Controls_Manager::SLIDER,
        'range' => [
          'em' => [
            'max' => 10,
            'step'=>1
          ],
          'px' => [
            'max' => 200,
            'step'=>1
          ],
        ],  
        'default'=>['size'=>1,'unit'=>'em'],
        'size_units' => [ 'px', 'em' ],
        'selectors' => [
          '{{WRAPPER}} .meta-position-bottom .meta-divider span' => 'font-size: {{SIZE}}{{UNIT}};',
        ],
        'condition' => [
          'bottom_meta_divider!' => ''
        ],
      ]
    );

    $this->add_control(
      'bottom_divider_color',
      [
        'label' => esc_html__( 'Color', 'elementor' ),
        'type' =>  Controls_Manager::COLOR,
        'default' => '',
        'selectors' => [
          '{{WRAPPER}} .meta-position-bottom .meta-divider span' => 'color: {{VALUE}};',
        ],
        'condition' => [
          'bottom_meta_divider!' => ''
        ],
      ]
    );

    $this->add_responsive_control(
        'bottom_divider_padding',
        [
            'label' => esc_html__( 'Padding', 'recent-posts-elementor-addon' ),
            'type' => Controls_Manager::DIMENSIONS,
            'size_units' => [ 'px', 'em' ],
            'default' => [],
            'selectors' => [
                '{{WRAPPER}} .meta-position-bottom .meta-divider' => 'padding: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
            ],
        ]
    );

    $this->end_controls_section();

    $this->start_controls_section(
      'post_readmore_style',
      [
        'label' => esc_html__( 'Readmore', 'recent-posts-elementor-addon' ),
        'tab'   => Controls_Manager::TAB_STYLE,
        'condition' => [
          'show_readmore!' => ''
        ],
      ]
    );  

    $this->add_responsive_control(
      'readmore_button_margin',
      [
          'label' => esc_html__( 'Margin', 'recent-posts-elementor-addon' ),
          'type' => Controls_Manager::DIMENSIONS,
          'size_units' => [ 'px', '%', 'em' ],
          'selectors' => [
              '{{WRAPPER}} .elementor-button' => 'margin: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
          ],
      ]
    );

    $this->add_control(
      'readmore_button_align',
      [
        'label' => esc_html__( 'Align', 'recent-posts-elementor-addon' ),
        'type' => Controls_Manager::CHOOSE,
        'options' => [
          'left' => [
            'title' => esc_html__( 'Left', 'recent-posts-elementor-addon' ),
            'icon' => 'eicon-text-align-left',
          ],
          'center' => [
            'title' => esc_html__( 'Center', 'recent-posts-elementor-addon' ),
            'icon' => 'eicon-text-align-center',
          ],
          'right' => [
            'title' => esc_html__( 'Right', 'recent-posts-elementor-addon' ),
            'icon' => 'eicon-text-align-right',
          ],
          'justify' => [
            'title' => esc_html__( 'Full Width', 'recent-posts-elementor-addon' ),
            'icon' => 'eicon-text-align-justify',
          ]
        ],
        'default' => '',
        'condition' => ['show_readmore[value]' => 'yes']
      ]
    );

    $this->add_control(
      'readmore_icon_indent',
      [
        'label' => esc_html__( 'Icon Spacing', 'recent-posts-elementor-addon' ),
        'type' => Controls_Manager::SLIDER,
        'range' => [
          'px' => [
            'max' => 100,
          ],
        ],
        'default' =>['value'=>5, 'unit'=>'px'],
        'selectors' => [
          '{{WRAPPER}} .elementor-button .elementor-align-icon-right' => 'margin-left: {{SIZE}}{{UNIT}};',
          '{{WRAPPER}} .elementor-button .elementor-align-icon-left' => 'margin-right: {{SIZE}}{{UNIT}};',
        ],
        'condition' => ['readmore_icon[value]!' => ''],
      ]
    );

    $this->add_group_control(
      Group_Control_Typography::get_type(),
      [
        'name' => 'typography_readmore_button',
        'selector' => '{{WRAPPER}} .elementor-button',
      ]
    );

    $this->add_responsive_control(
      'readmore_button_padding',
      [
        'label' => esc_html__( 'Padding', 'recent-posts-elementor-addon' ),
        'type' => Controls_Manager::DIMENSIONS,
        'size_units' => [ 'px', 'em', '%' ],
        'selectors' => [
          '{{WRAPPER}} .elementor-button' => 'padding: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
        ]
      ]
    );
    
    $this->add_control(
      'readmore_button_radius',
      [
        'label' => esc_html__( 'Border Radius', 'recent-posts-elementor-addon' ),
        'type' => Controls_Manager::DIMENSIONS,
        'size_units' => [ 'px', '%' ],
        'selectors' => [
          '{{WRAPPER}} .elementor-button' => 'border-radius: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
        ]
      ]
    );

    $this->add_group_control(
      Group_Control_Border::get_type(),
      [
        'name' => 'readmore_button_border',
        'selector' => '{{WRAPPER}} article .elementor-button',
        'separator' => 'before',
      ]
    );


    $this->start_controls_tabs( 'tabs_readmore_button_style' );

    $this->start_controls_tab(
      'tab_readmore_button_normal',
      [
        'label' => esc_html__( 'Normal', 'recent-posts-elementor-addon' ),
      ]
    );


    $this->add_control(
      'readmore_button_color',
      [
        'label' => esc_html__( 'Color', 'recent-posts-elementor-addon' ),
        'type' => Controls_Manager::COLOR,
        'default' => '',
        'selectors' => [
          '{{WRAPPER}} .elementor-button' => 'fill: {{VALUE}}; color: {{VALUE}};',
        ],
      ]
    );

    $this->add_control(
      'readmore_button_background',
      [
        'label' => esc_html__( 'Background', 'recent-posts-elementor-addon' ),
        'type' => Controls_Manager::COLOR,
        'selectors' => [
          '{{WRAPPER}} .elementor-button' => 'background-color: {{VALUE}};',
        ],
      ]
    );

    $this->end_controls_tab();

    $this->start_controls_tab(
      'tab_readmore_button_hover',
      [
        'label' => esc_html__( 'Hover', 'recent-posts-elementor-addon' ),
      ]
    );


    $this->add_control(
      'readmore_button_hover_color',
      [
        'label' => esc_html__( 'Color', 'recent-posts-elementor-addon' ),
        'type' => Controls_Manager::COLOR,
        'selectors' => [
          '{{WRAPPER}} .elementor-button:hover, {{WRAPPER}} .elementor-button:focus' => 'color: {{VALUE}}!important;',
          '{{WRAPPER}} .elementor-button:hover svg, {{WRAPPER}} .elementor-button:focus svg' => 'fill: {{VALUE}}!important;',
        ],
      ]
    );

    $this->add_control(
      'readmore_button_background_hover_color',
      [
        'label' => esc_html__( 'Background', 'recent-posts-elementor-addon' ),
        'type' => Controls_Manager::COLOR,
        'selectors' => [
          '{{WRAPPER}} .elementor-button:hover, {{WRAPPER}} .elementor-button:focus' => 'background-color: {{VALUE}};',
        ],
      ]
    );

    $this->add_control(
      'readmore_button_hover_border_color',
      [
        'label' => esc_html__( 'Border', 'recent-posts-elementor-addon' ),
        'type' => Controls_Manager::COLOR,
        'condition' => [
          'readmore_button_border_border!' => '',
        ],
        'selectors' => [
          '{{WRAPPER}} .elementor-button:hover, {{WRAPPER}} .elementor-button:focus' => 'border-color: {{VALUE}};',
        ],
      ]
    );

    $this->end_controls_tab();
    $this->end_controls_tabs();


    $this->end_controls_section();

  }

  protected function render() {

    $settings = $this->get_settings_for_display();

    extract( $settings );

    $query_params = $this->get_post_query($settings);

    $post_query = new WP_Query($query_params);

    if (is_wp_error($post_query) || !$post_query->have_posts()) {
      return '';
    }

    $rows_html  = $frontend_settings = array();

    while ( $post_query->have_posts() ) : 

     $post_query->the_post();
     $rows_html[] = $this->get_post($settings);

    endwhile;

    wp_reset_postdata();

    $attribute = '';

    $col_class = 'elementor-element post-list image-position-'.$image_position;

    if($list_animation !=''){


      if(is_admin()){
        $col_class.= ($list_animation_duration!='' ? ' animated-'.$list_animation_duration: '').' animated';
      }else{

        $col_class.= ($list_animation_duration!='' ? ' animated-'.$list_animation_duration: '').' elementor-invisible'; 

      $frontend_settings['_animation']= $list_animation;
      $frontend_settings['_animation_delay']= $list_animation_delay;
      $attribute = 'data-settings='.wp_json_encode( $frontend_settings );

      }
    }

    if($image_totop === 'yes'){
      $col_class.=' mobile-force-ontop';
    }
    echo '<div class="gum_posts"><div class="'.$col_class.'" data-element_type="widget" '.esc_attr($attribute).'>'.join('</div><div class="'.$col_class.'" data-element_type="widget" '.esc_attr($attribute).'>',$rows_html).'</div></div>';

  }

  protected function get_post($settings = array()) {

    if(!isset( $settings ) || empty( $settings )){
      $settings = $this->get_settings_for_display();
    }

    extract( $settings );      

    $post_id = get_the_ID();
    $post_title = get_the_title();

    $hide_content = true; $post_content = $image_url = '';

    if( $show_excerpt =='yes'){

      $post_content = strip_shortcodes( get_the_excerpt() );
      if($post_content_word !='' && $post_content_word > 0){
        $post_content = wp_trim_words($post_content ,  absint($post_content_word) , $post_content_sufix );
      }

    }

    $this->add_toprevious_blog( $post_id );
    $post_url = get_the_permalink( $post_id );

    if($show_image ==='yes'){

      $thumb_id = get_post_thumbnail_id( $post_id );
      $image = ['id' => $thumb_id ];
      $settings['thumbnail'] = $image;

      $image_url = Group_Control_Image_Size::get_attachment_image_src( $thumb_id , 'thumbnail', $settings);

      if ( ! empty( $image_url ) ) {
        $image_html = sprintf( '<img src="%s" title="%s" alt="%s" />', esc_attr( $image_url ), Control_Media::get_image_title( $thumb_id ), Control_Media::get_image_alt( $thumb_id ) );
      }

    }

    $allowed_tags = array('h1','h2','h3','h4','h5','h6','div');
    $tag_title = (in_array( $tag, $allowed_tags )) ? trim( $tag ): 'h4';

    ob_start();
?>
<article id="post-<?php print esc_attr($post_id); ?>" <?php post_class(); ?>>
    <?php if($image_url!=''):?>
  <div class="post-top">
    <div class="blog-image" style="background-image: url('<?php print $image_url; ?>');"><?php print $image_html;?></div>
  </div>
    <?php endif;?>
  <div class="post-content">
      <?php $this->get_post_meta( $settings , 'top') ?>
      <?php printf( '<a href="%s"><%s class="post-title">%s</%s></a>', $post_url ,$tag_title, esc_html( $post_title ), $tag_title);?>
      <?php $this->get_post_meta( $settings , 'mid'); ?>
      <?php if($show_excerpt === 'yes'){ printf( '<div class="content-excerpt clearfix">%s</div>', $post_content); } ?>
      <?php $this->get_post_meta( $settings , 'bottom'); ?>
      <?php $this->get_readmore_button( 'post-'.$post_id,  $settings , $post_url); ?>
  </div>
</article>
<?php 

    return ob_get_clean();
  }

  protected function get_post_meta($settings = array(), $position='') {

    if( empty($position) ) return '';

    $key = trim($position).'_meta';
    $spacer ='';

    if(empty( $settings[ $key ] )) return '';

     $spacer = '';

     if(isset($settings[ $position.'_meta_divider'])){

        if($settings[ $position.'_meta_divider'] == 'text'){
          $spacer = '<span>'.$settings[ $position.'_divider_text'].'</span>';

        }elseif($settings[ $position.'_meta_divider'] == 'icon'){
          ob_start();
          
          Icons_Manager::render_icon( $settings[ $position.'_divider_icon'], ['aria-hidden' => 'true'],'span' );

          $spacer = ob_get_clean();
        }
     }

    $post_id = get_the_ID();

    $metas = $settings[ $key ];
    $rows_html  = array();

    foreach ($metas as $meta) {

        $meta_url = '';
        $meta_type = '';


        switch ($meta) {
          case 'date_meta':
            $meta_type = get_the_date();

            $rows_html[] = '<li class="list-meta">'. ( $meta_url !='' ? sprintf('<a href="%s"><span class="meta-text">%s</span></a>', $meta_url, $meta_type) : sprintf('<span class="meta-text">%s</span>',$meta_type) ).'</li>';

            break;
          case 'author_meta':

            $author_id = get_post_field( 'post_author', $post_id );
            $meta_type = get_the_author_meta('nickname', $author_id);
            $meta_url = get_the_author_meta('url',$author_id);

            $rows_html[] = '<li class="list-meta">'. ( $meta_url !='' ? sprintf('<a href="%s"><span class="meta-text">%s</span></a>', $meta_url, $meta_type) : sprintf('<span class="meta-text">%s</span>',$meta_type) ).'</li>';

            break;
          case 'category_meta':
          case 'categories_meta':
            $categories = get_the_category($post_id);

            if($categories){

              foreach ($categories as $category) {

                  $meta_type = $category->name;
                  $meta_url = get_category_link( $category->term_id );

                  $rows_html[] = '<li class="list-meta">'. ( $meta_url !='' ? sprintf('<a href="%s"><span class="meta-text">%s</span></a>', $meta_url, $meta_type) : sprintf('<span class="meta-text">%s</span>',$meta_type) ).'</li>';

                  if($meta =='category_meta') break;

              }

            }
            break;
          default:
            $meta_type = '';
            break;
        }

    }

    echo '<ul class="meta-position-'.esc_attr($position).' posts-meta" style="list-style:none;">'.join('<li class="meta-divider">'.$spacer.'</li>',$rows_html).'</ul>';

  }

  protected function get_post_query($settings = array()) {


    if(!isset( $settings ) || empty( $settings )){
      $settings = $this->get_settings_for_display();
    }

    extract( $settings );  

    $posts_per_page = isset($posts_per_page) && $posts_per_page!='' ? $posts_per_page : 10;

    $query_params= array(
      'posts_per_page' => $posts_per_page,
      'no_found_rows' => false,
      'post_status' => 'publish',
      'post_type'=>'post',
      'paged' => 0,
      'ignore_sticky_posts' => true
    );

    $meta_query= $excludes = array();

    if( $posts_offset && $posts_offset > 0 ){
      $query_params['offset'] = absint( $posts_offset );
    }

    if( $skip_previous === 'yes'){
      $excludes =  $this->get_previous_blog();
    }

    if($single_exclude === 'yes' && is_single()){
      $excludes[] = $post_id = get_the_ID();
    }

    if(count($excludes)){
      $query_params['post__not_in'] = $excludes;
    }

    if($is_featured_image === 'yes'){
      $meta_query['relation'] = 'AND';
      $meta_query[] = array(
        'key' => '_thumbnail_id'
        );
    }

    switch ($filter_by) {
      case 'category':

            if($cat_ids == 'only'){

                $post_id =  isset($post_id) ? $post_id : get_the_ID(); 
                $cat_options= [];

                if( $categories= get_the_category($post_id)){
                  foreach ($categories as $category) {
                    $cat_options[]= $category->term_id;
                  }

                }

                $query_params['cat'] = $cat_options;

            }elseif($cat_ids !=''){
              $query_params['cat']= trim($cat_ids);
            }
        break;      
      case 'tag':
            if($tags_ids !=''){

              $query_params['tax_query'] = array(
                array(
                  'taxonomy' => 'post_tag',
                  'field' => 'id',
                  'terms' => array( $tags_ids ),
                  'operator' => 'IN'
                )
              );
            }
        break;
      case 'ids':
            if($posts_ids !=''){
              $query_params['post__in']=explode(",",$posts_ids);
            }
        break;
      case 'sticky':
              $query_params['ignore_sticky_posts']= false;
              $sticky_posts = get_option('sticky_posts');

              if( isset($query_params['post__not_in']) && !empty( $query_params['post__not_in'] )){
                $excludes_post = $query_params['post__not_in'];

                foreach ($sticky_posts as $k => $value) {
                  if(in_array($value, $excludes_post)) unset($sticky_posts[$k]);
                }
              }

              $query_params['post__in']= count( $sticky_posts ) ? $sticky_posts : array(0);

        break;
      default:
        break;
    }

    if($source_orderby=='comment'){
      $query_params['orderby'] = 'comment_count';
      $query_params['order'] = 'DESC';
    }

    if(count($meta_query)){
      $query_params['meta_query'] = $meta_query;
    }

    return $query_params;

  }
  
  protected function get_readmore_button( $index, $settings = array(), $url ='' ) {

     if($settings['show_readmore']!='yes' ) return '';

    $this->add_render_attribute( 'button-'.$index ,
      [
        'class' => ['elementor-button', 'readmore-button' ],
        'role' => 'button'
      ]
    );

    $this->add_link_attributes( 'button-'.$index, array('url' => $url ) );
    $this->add_render_attribute( 'button-'.$index, 'class', 'elementor-button-link' );

    $this->add_render_attribute( [
      'readmore_icon_align' => [
        'class' => [
          'elementor-button-icon',
          'elementor-align-icon-' . $settings['readmore_icon_align'],
        ],
      ],
    ] );

    $this->add_render_attribute( $index , 'class', 'elementor-button-text' );
    $this->add_inline_editing_attributes( $index, 'none' );

    ?><div class="elementor-button-wrap<?php print ' button-align-'.$settings['readmore_button_align'] ;?>"><a <?php echo $this->get_render_attribute_string( 'button-'.$index ); ?>>
          <span class="elementor-button-content-wrapper">
      <?php if ( ! empty( $settings['readmore_icon']['value'] ) ) : ?>
      <span <?php echo $this->get_render_attribute_string( 'readmore_icon_align' ); ?>>
          <?php Icons_Manager::render_icon( $settings['readmore_icon'], [ 'aria-hidden' => 'true' ] ); ?>
      </span>
      <?php endif; ?>
      <span <?php echo $this->get_render_attribute_string( $index );?>><?php echo $settings['readmore_label']; ?></span>
    </span>
  </a></div><?php

  }

  public function get_previous_blog(  ) {

    global $gum_helper;

    if(!isset($gum_helper) || !isset( $gum_helper['blog_ids'] )){
      $gum_helper['blog_ids'] = array();
    }

    return $gum_helper['blog_ids'];

  } 


  public function add_toprevious_blog( $ids ) {

    global $gum_helper;

    $previous_blog = $this->get_previous_blog();

    if( is_array($ids) ){
      $previous_blog = array_merge( $previous_blog, $ids );
    }
    else{
      array_push($previous_blog, $ids );
    }

    $gum_helper['blog_ids'] = array_unique( $previous_blog );

  }

  public function enqueue_script( ) {

    wp_enqueue_style( 'recent-posts-elementor-addon', GUM_RCPOST_ELEMENTOR_URL."css/style.css",array());
  }


}

// Register widget
\Elementor\Plugin::instance()->widgets_manager->register_widget_type( new Gum_Elementor_Widget_Recent_posts() );

?>
